/*
m_mixvec_a - Mix down audio signals of a vector into one output

DESCRIPTION
This opcode takes an array of audio signals and outputs the mix of any two
adjecent elements - or channels - into the output, like a sequential switch
with blending or interpolation.

SYNTAX
aOutput m_mixvec_a aInputs[], kMix, [iMode, iWrap, iBlend]

INITIALIZATION
iMode (optional) - Indexing mode for the mix control (default: 0).
	0: kMix goes from 0 to number of inputs - 1
	1: kMix is normalised to a range of 0 to 1
iWrap (optional) - Wrap around control (default: 1)
	0: Don't wrap around, stop at maximum and minimum index
	1: wrap around, so the input behaves more like a ring than a vector
iBlend (optional) - Whether to switch or blend between the inputs (default: 1)
	0: Discretely switch between the inputs
	1: Seamlessly blend between the inputs

PERFORMANCE
aOutput - Output audio signal
aInputs[] - Array of input audio signals
kMix - Mix control, depending on iMode this is in the range of 0 to 1 or
	0 to number of outputs - 1

CREDITS
Author: Jeanette C.
*/

opcode m_mixvec_a, a, a[]kopp
	aIn[], kMix, iMode, iWrap, iBlend xin

	; Initialise internal parameters
	aOut init 0 ; output 
	iOuts = lenarray(aIn)
	kRealMix init 0 ; internal mix control
	kIndex init 0 ; lower element index
	kVol init 1 ; volume;/balance of the higher input
	iHiMix = (iOuts - 1) ; maximum index

	; Check the indexing mode and set the internal rescale factor
	if (iMode == 0) then ; between 0 and n-1
		iFactor init 1
	else ; between 0 and 1
		iFactor init (iOuts -1)
	endif

	; Process the mix value according mode and wrap state
	if (iWrap == 0) then
		kRealMix = limit((kMix * iFactor), 0, iHiMix)
	else
		kRealMix = wrap((kMix * iFactor), 0, (iHiMix + .999))
	endif

	; Set up the index and balance
	kIndex = int(kRealMix)
	if (kIndex < iHiMix) then
		kVol = frac(kRealMix)
	else ; above maximum index
		if (iWrap == 0) then ; if wrap is off stop at last index on max
			kVol = 0
		else
			kVol = frac(kRealMix)
		endif
	endif

	; Set the output value
	if (iBlend == 0) then
		aOut = aIn[kIndex]
	else
		aOut = aIn[kIndex] * (1 - kVol)
		if (kIndex < iHiMix) then
			aOut += aIn[kIndex + 1] * kVol
		else
			if (iWrap == 1) then
				aOut += aIn[0] * kVol
			endif
		endif
	endif

	; Control output
	xout(aOut)
endop
