/*
m_vec_mix_a - A vector mixer for audio signals

DESCRIPTION
m_vec_mix_a will mix one audio input signal to certain positions of
the output array, based on the kMix input, not dissimilar to what the vbap
opcodes do.

SYNTAX
aOut[] m_vec_mix_a aInput, kMix, iNumberOfOutputs, [iMode, iWrap, iBlend]

INITIALIZATION
iNumberOfOutputs - The number of elements (mixer channels) the output array
	should have
iMode (optional) - Indexing mode, defaults to 0
	0: kMix ranges from 0 to Number of outputs - 1
	1: kMix ranges from 0 to 1
iWrap (optional) - Wrap around mode, defaults to 1.
	0: Don't wrap, kMix will be limited internally to avoid out-of-range
		errors
	1: Wrap around, kMix is wrapped internally, so the output vector acts more
		like a ring, with indices above max, distributing the signal between
		the last output element and the first
iBlend (optional) - Whether to smealessly blend or discretely switch between
	the outputs. Defaults to 1
	0: Discretely switch between the outputs
	1: Seamlessly blend between the outputs

PERFORMANCE
aOutput[] - Output array, can be seen as an array of output channels of the
	mixer
aInput - The input signal to be distributed among the output channels

CREDITS
Author: Jeanette C.
*/

opcode m_vecmix_a, a[], akiopp ; iMode=0: kmix 0-n, iMode!=0: kmix 0-1
	aIn, kMix, iOuts, iMode, iWrap, iBlend xin

	; Set up internal controls
	aOut[] init iOuts
	kRealMix init 0 ; internal mix control
	kIndex init 0 ; lower vector index
	kLastIndex init 0 ; previous lower vector index
	kVol init 1 ; volume/balance of higher output
	iHiMix init (iOuts - 1) ; maximum value for internal mix balance

	; Check the indexing/mix mode and set the internal rescale factor
	if (iMode == 0) then ; between 0 and n-1
		iFactor init 1
	else
		iFactor init (iOuts -1)
	endif

	; Process the mix value according mode and wrap state
	if (iWrap == 0) then
		kRealMix = limit((kMix * iFactor), 0, iHiMix)
	else
		kRealMix = wrap((kMix * iFactor), 0, (iHiMix + .999))
	endif

	; Set up the index and balance
	kIndex = int(kRealMix)
	if (kIndex < iHiMix) then
		kVol = frac(kRealMix)
	else ; above maximum output
		if (iWrap == 0) then ; if wrap is off stop at last output on max
			kVol = 0
		else
			kVol = frac(kRealMix)
		endif
	endif

	if (kIndex != kLastIndex) then ; set old output elements to 0
		aOut[kLastIndex] = 0
		if (kLastIndex < iHiMix) then
			aOut[kLastIndex + 1] = 0
		else
			if (iWrap == 1) then
				aOut[0] = 0
			endif
		endif
	endif

	; Set the output values
	if (iBlend == 0) then
		aOut[kIndex] = aIn
	else
		aOut[kIndex] = aIn * (1 - kVol)
		if (kIndex < iHiMix) then
			aOut[kIndex + 1] = aIn * kVol
		else
			if (iWrap == 1) then
				aOut[0] = aIn * kVol
			endif
		endif
	endif

	; Update the last index
	kLastIndex = kIndex

	; Audio output
	xout(aOut)
endop
