/*
m_vec_mix_k - A vector mixer for control signals

DESCRIPTION
m_vec_mix_k will mix one control input signal to certain positions of
the output array, based on the kMix input, not dissimilar to what the vbap
opcodes do for audio.

SYNTAX
kOut[] m_vec_mix_k kInput, kMix, iNumberOfOutputs, [iMode, iWrap, iBlend]

INITIALIZATION
iNumberOfOutputs - The number of elements (mixer channels) the output array
	should have
iMode (optional) - Indexing mode, defaults to 0
	0: kMix ranges from 0 to Number of outputs - 1
	1: kMix ranges from 0 to 1
iWrap (optional) - Wrap around mode, defaults to 1.
	0: Don't wrap, kMix will be limited internally to avoid out-of-range
		errors
	1: Wrap around, kMix is wrapped internally, so the output vector acts more
		like a ring, with indices above max, distributing the signal between
		the last output element and the first
iBlend (optional) - Whether to blend between the signal or switch, defaults
	to 1.
	0: Discretely switch between the outputs
	1: Seamlessly blend between the outputs

PERFORMANCE
kOutput[] - Output array, can be seen as an array of output channels of the
	mixer
kInput - The input signal to be distributed among the output channels

CREDITS
Author: Jeanette C.
*/

opcode m_vecmix_k, k[], kkiopp ; iMode=0: kmix 0-n, iMode!=0: kmix 0-1
	kIn, kMix, iOuts, iMode, iWrap, iBlend xin

	; Set up internal controls
	kOut[] init iOuts
	kRealMix init 0 ; internal mix control
	kIndex init 0 ; lower vector index
	kLastIndex init 0 ; previous lower vector index
	kVol init 1 ; volume/balance of higher output
	iHiMix init (iOuts - 1) ; maximum value for internal mix balance

	; Check the indexing/mix mode and set the internal rescale factor
	if (iMode == 0) then ; between 0 and n-1
		iFactor init 1
	else
		iFactor init (iOuts -1)
	endif

	; Process the mix value according mode and wrap state
	if (iWrap == 0) then
		kRealMix = limit((kMix * iFactor), 0, iHiMix)
	else
		kRealMix = wrap((kMix * iFactor), 0, (iHiMix + .999))
	endif

	; Set up the index and balance
	kIndex = int(kRealMix)
	if (kIndex < iHiMix) then
		kVol = frac(kRealMix)
	else ; above maximum output
		if (iWrap == 0) then ; if wrap is off stop at last output on max
			kVol = 0
		else
			kVol = frac(kRealMix)
		endif
	endif

	if (kIndex != kLastIndex) then ; set old output elements to 0
		kOut[kLastIndex] = 0
		if (kLastIndex < iHiMix) then
			kOut[kLastIndex + 1] = 0
		else
			if (iWrap == 1) then
				kOut[0] = 0
			endif
		endif
	endif

	; Set the output values
	if (iBlend == 0) then
		kOut[kIndex] = kIn
	else
		kOut[kIndex] = kIn * (1 - kVol)
		if (kIndex < iHiMix) then
			kOut[kIndex + 1] = kIn * kVol
		else
			if (iWrap == 1) then
				kOut[0] = kIn * kVol
			endif
		endif
	endif

	; Update the last index
	kLastIndex = kIndex

	; Control output
	xout(kOut)
endop
