/*
m_vecrand_k - randomly map control inputs from one vector to another

DESCRIPTION
Randomly map the signals of an input vector to an output vector.

SYNTAX
kOutputs[] m_vecrand_k kInputs[], kFreq, kAmount, [iMode, iBlend]

INITIALIZATION
iMode (optional) - Interpretation of amount (default: 1)
	0: amount ranges from 0 to number of inputs - 1
	1: amount ranges from 0 to 1
iBlend (optional) - Whether to blend or discretely switch between states
	(default: 1)
	0: Discretely switch
	1: Seamlessly blend

PERFORMANCE
kOutputs[] - Output vector
kInputs[] - Input vector
kFreq - Frequency of state change in Hz
kAmount - Amount of randomness (see iMode above for the range)

CREDITS
Author: Jeanette C.
*/

opcode m_vecrand_k, k[], k[]kkpp
	kInputs[], kFreq, kAmount, iMode, iBlend xin

	; Initialise internal parameters
	iIns = lenarray(kInputs)
	kOutputs[] init iIns
	kIndices[] init iIns ; array of current random positions
	kOldIndices[] init iIns ; internal array holding old random indices
	aSyncDummy init 0 ; dummy input for syncphasor
	aPhase, aSync syncphasor kFreq, aSyncDummy ; change trigger and blender
	kIndex init 0 ; internal index
	kNewPos init 0 ; internal momentary new position
	iHiMix init (iIns - 1) ; Maximum index
	kRealAmount init 0 ; internal variation amount
	kVol init 0 ; volume/balance of the higher input
	kVal init 0 ; Momentary current signal value
	kOldVal init 0 ; Momentary signal value of previous state

	; Initialise the kOldIndices array with fixed beginning state
	iIndex init 0
	while (iIndex < iIns) do
		kOldIndices[iIndex] init iIndex
		iIndex += 1
	od

	; Set the amount scale factor based on iMode
	if (iMode == 0) then
		iFactor init 1
	else
		iFactor init iHiMix
	endif

	kRealAmount = kAmount * iFactor

	; Generate new random indices
	if (k(aSync) != 0) then ; trigger new distribution
		kIndex = 0
		while (kIndex < iIns) do
			kOldIndices[kIndex] = kIndices[kIndex] ; copy old state
			kNewPos = wrap((linrand:k(2 * kAmount) - kAmount + kIndex), 0, (iHiMix + .9999))
			kIndices[kIndex] = kNewPos
			kIndex += 1
		od
	endif

	; Create output
	if (iBlend == 0) then ; don't interpolate with previous distribution
		kIndex = 0
		while (kIndex < iIns) do
			kNewPos = int(kIndices[kIndex])
			kNextPos = wrap((kNewPos + 1), 0, (iHiMix + .1))
			kVol = frac(kIndices[kIndex])
			kOutputs[kIndex] = (kInputs[kNewPos] * (1 - kVol)) + (kInputs[kNextPos] * kVol)
			kIndex += 1
		od
	else ; Blend with old values
		kIndex = 0
		while (kIndex < iIns) do
			; Calculate value of current state with blend
			kNewPos = int(kIndices[kIndex])
			kNextPos = wrap((kNewPos + 1), 0, (iHiMix + .1))
			kVol = frac(kIndices[kIndex])
			kVal = (kInputs[kNewPos] * (1 - kVol)) + (kInputs[kNextPos] * kVol)
			kVal *= k(aPhase)

			; Calculate value of old state with blend
			kNewPos = int(kOldIndices[kIndex])
			kNextPos = wrap((kNewPos + 1), 0, (iHiMix + .1))
			kVol = frac(kOldIndices[kIndex])
			kOldVal = (kInputs[kNewPos] * (1 - kVol)) + (kInputs[kNextPos] * kVol)
			kOldVal *= (1 - k(aPhase))

			; Write current blended value to output
			kOutputs[kIndex] = kVal + kOldVal
			kIndex += 1
		od
	endif

	; Control output
	xout(kOutputs)
endop
