/*
m_vecshift_a - vector shifter for audio signals

DESCRIPTION
Mix an input array of signals to an output array with the ability to shift/move
the inputs around, so they get mixed to different outputs.

SYNTAX
aOutputs[] m_vecshift_a aInputs[], kPosition, [iMode, iBlend]

INITIALIZATION
iMode (optional) - Indexing mode (default: 0)
	0: kPosition ranges from 0 to number of inputs - 1
	1: kPosition ranges from 0 to 1
iBlend (optional) - Blend or switch mode (default: 1)
	0: switch discretely
	1: seamlessly blend

PERFORMANCE
aOutputs[] - Output signal vector
aInputs[] - Input signal vector
kPosition - Shift offset/position, the position is always wrapped round

CREDITS
Author: Jeanette C.
*/

opcode m_vecshift_a, a[], a[]kop
	aInputs[], kPosition, iMode, iBlend xin

	; Initialise internal parameters
	iIns = lenarray(aInputs) ; number of input/output elements
	aOutputs[] init iIns ; Output array
	kRealPos init 0 ; Internal position control
	kOffset init 0 ; offset position of shift
	kIndex init 0 ; Momentary position index
	kVol init 0 ; Higher element volume
	iHiMix init (iIns - 1) ; maximum value for internal mix balance

	; Check the indexing/mix mode and set the internal rescale factor
	if (iMode == 0) then ; between 0 and n-1
		iFactor init 1
	else
		iFactor init (iIns -1)
	endif

	; Set the internal position
	kRealPos = wrap((kPosition * iFactor), 0, (iHiMix + .999))
	kOffset = int(kRealPos)
	kVol = frac(kRealPos)

	; Create the output
	if (iBlend == 0) then ; plain shifting
		kIndex = 0
		while (kIndex < iIns) do
			kInIndex = wrap((kIndex + kOffset), 0, (iHiMix + .1))
			aOutputs[kIndex] = aInputs[kInIndex]
			kIndex += 1
		od
	else ; blending
		kIndex = 0
		while (kIndex < iIns) do
			kInIndex = wrap((kIndex + kOffset), 0, (iHiMix + .1))
			kInNext = wrap((kInIndex + 1), 0, (iHiMix + .1))
			aOutputs[kIndex] = (aInputs[kInIndex] * (1 - kVol)) + (aInputs[kInNext] * kVol)
			kIndex += 1
		od
	endif

	; Audio output
	xout(aOutputs)
endop
